<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use App\Exception;
use App\Exception\ApiException as ApiE;

use App\Templates;

class WebService {
    
    const API_URL = "https://deliveryapp.neemo.com.br/api/integration/v1/";
    const FAILOVER_EMAIL = "daniel.fonseca@teknisa.com";
    
    private $connection;
    private $integrationService;
    private $consumerService;
    private $util;
    private $dispatchService;
    private $mailList;
    private $accessToken;
    private $appToken;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Service\IntegrationService $integrationService,
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Util\Util $util,
                                 \App\Service\DispatchService $dispatchService)
    {
        
        $this->connection         = $connection;
        $this->integrationService = $integrationService;
        $this->consumerService    = $consumerService;
        $this->util               = $util;
        $this->dispatchService    = $dispatchService;
    }


    public function runWebService(Request $request, Response $response)
    {
        try {
            $req = $request->getParsedBody();
            
            if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") 
                throw new \Exception("Auth inválido.", 666);

            $configuration = $this->integrationService->getConfiguration();

            if(empty($configuration)) {
                $response->withStatus(200);
                return 0;
            }
            $ufTimezone      = (isset($configuration['IDUFTIMEZONE']) && !empty($configuration['IDUFTIMEZONE'])) ? $configuration['IDUFTIMEZONE'] : "SP";
            $trackingEnabled = (isset($configuration['IDRASTREIAENTREGA']) && ($configuration['IDRASTREIAENTREGA'] == "S"));

            date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));

            if($configuration['IDHABINTE'] == 'N') {
                $response->withStatus(200);
                return 0;
            }
            
            $this->mailList  = $this->util->getMailList();
            
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $clientFolder = $this->connection->getParams();
            $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

            $this->basePath          = $config['logPath'] . "/$clientFolder";
            $this->ordersPath        = $this->basePath . "/orders";
            $this->processedPath     = $this->basePath . "/processed";
            $this->errPath           = $this->basePath . "/error";
            $this->networkErrPath    = $this->basePath . "/networkError";
            $this->serverName        = $config['server'];
            
            if(empty($configuration['IDTOKENINTGERAL'])) {
                throw new Exception\GeneralErrorException("Token geral não encontrado");
            }
            
            $generalToken = $configuration['IDTOKENINTGERAL'];

            if(empty($configuration['IDSENHATOKENGERAL'])) {
                throw new Exception\GeneralErrorException("App Token não encontrado");
            }
            
            $this->appToken = $configuration['IDSENHATOKENGERAL'];
            
            $yesterday = date('Y-m-d H:i:s', strtotime('-1 day'));

            $orderData = json_decode($this->executeWebService("order/master/orders", "POST", array("status" => 0, "created_at" => $yesterday), $generalToken), true);

            if($orderData["code"] != 200) 
                throw new ApiE($orderData["error"], $orderData["code"]);

            if (!count($orderData["Orders"])) {
                if($trackingEnabled) $this->dispatch();
                
                $response->withStatus(200);
                return 0;
            }
            
            foreach ($orderData["Orders"] as $order) {
                
                
                ////////////// ORDER FILTER ///////////////
                
                if(empty($order["account_ref"])) {
                    continue;// skip orders with ref null
                }
                    
                if (time() - strtotime($order["date"]) > 28800) {
                    continue;// skip orders olders than 8 hours
                }
                    
                ////////////// ORDER FILTER ///////////////
                
                $this->connection->beginTransaction();

                try {
                    $fileName =  "INDEFINIDO-" . date("H-i-s") . ".json";

                    $cdFilial = $order["account_ref"];
                    
                    $individualOrder = json_decode($this->executeWebService("order/".$order["id"], "POST"), true);
                    $idOrder = $individualOrder['Order']['id'];

                    $fileName = (isset($idOrder) ? ("$cdFilial-{$idOrder}") : ("INDEFINIDO-" . date("H-i-s"))) . ".json"; 
                    
                    $this->util->writeLog('order', $fileName, $individualOrder);
                    
                    if($individualOrder["code"] != 200) 
                        throw new ApiE("Error", $individualOrder["code"]);
                    
                    $individualOrder["Order"]["branch"] = $order["account_ref"];

                    $comd = $this->processOrder($individualOrder["Order"]);
                    
                    $this->integrationService->validateCashierSinc($comd["CDFILIAL"]);
                    
                    $this->integrationService->validateOrder($comd['CDFILIAL'], $comd['NRVENDAREST']);
                    
                    $this->connection->commit();
                    
                    if($trackingEnabled)
                        $this->dispatchService->registerPendingDispatchOrder($comd["CDFILIAL"], $comd["NRCOMANDA"], $comd["NRCOMANDAEXT"], $order["id"]);

                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Pedido - " . $order["id"] . " Integrado com sucesso\n");
                    $this->executeWebService("order/".$order["id"], "PUT", array("status" => "1"));
                    
                    $attOrder = json_decode($this->executeWebService("order/".$order["id"], "POST"), true);
                    $this->util->writeLog('order_processed', $fileName, $attOrder);
                    
                }
                catch (ApiE $e) {
                    $this->connection->rollback();

                    $msg = "Não foi possível integrar o pedido - ".$order["id"]."; Motivo: Erro na requisição à API. ErrCode: " . $e->getCode() . " - Message: " . $e->getMessage().PHP_EOL;
                    $order['msg'] = $msg;
                    $this->util->writeLog("execution", "", $msg);

                    $this->executeWebService("order/".$order["id"], "PUT", array("status" => "3"));
                    $this->util->writeLog('order_error', $fileName, $order);
                    
                }
                catch (Exception\ParamsException $e) {
                    $this->connection->rollback();

                    $msg = date("d/m/Y H:i:s") . " Não foi possível integrar o pedido - " .$order["id"].PHP_EOL;
                    $msg .= "- Motivo: Erro na requisição à API. ErrCode: " . $e->getCode().PHP_EOL;
                    $msg .= "- Message: " . $e->getMessage().PHP_EOL;
                    
                    $order['msg'] = $msg;

                    $this->executeWebService("order/".$order["id"], "PUT", array("status" => "3"));
                    $this->util->writeLog('order_error', $fileName, $order);
                    $this->util->writeLog('execution', "", $msg);
                    $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                    $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Neemo", $template, [
                        ":INTEGRACAO"    => "Neemo",
                        ":NUMERO_PEDIDO" => "#".$order["account_ref"]."-".$order["order_number"],
                        ":NOME_CLIENTE"  => $this->getClientName(),
                        ":NOME_SERVIDOR" => $this->serverName,
                        ":ERRO_MSG"      => $e->getMessage(),
                        ":JSON_PEDIDO"   => json_encode($individualOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                        ":GERADO_EM"     => date("d/m/Y H:i:s")
                    ]);
                }
                catch (\Exception $e) {
                    $this->connection->rollback();

                    $msg = date("d/m/Y H:i:s") . " Não foi possível integrar o pedido - " .$order["id"].PHP_EOL;
                    $msg .= "- Motivo: Erro na requisição à API. ErrCode: " . $e->getCode().PHP_EOL;
                    $msg .= "- Message: " . $e->getMessage().PHP_EOL;

                    $order['msg'] = $msg;
                    
                    $this->executeWebService("order/".$order["id"], "PUT", array("status" => "3"));
                    
                    $this->util->writeLog('order_error', $fileName, $order);
                    $this->util->writeLog('execution', "", $msg);
                    
                    $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                    $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Neemo", $template, [
                        ":INTEGRACAO"    => "Neemo",
                        ":NUMERO_PEDIDO" => $order["order_number"],
                        ":NOME_CLIENTE"  => $this->getClientName(),
                        ":NOME_SERVIDOR" => $this->serverName,
                        ":ERRO_MSG"      => $e->getMessage(),
                        ":JSON_PEDIDO"   => json_encode($individualOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                        ":GERADO_EM"     => date("d/m/Y H:i:s")
                    ]);}
            }
        }
        catch (ApiE $e) {
            $this->util->writeLog("execution", "", "Erro na requisição à API. ErrCode: " . $e->getCode() . " - Message: " . $e->getMessage().PHP_EOL);
        }
        
        catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de rede (NetworkException):  {$e->getMessage()}\n");
        }
        
        catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." " . $e->getMessage() . "\n");
            $this->integrationService->disableIntegration();
            
            $template = file_get_contents(__DIR__ ."/../../templates/general-error.html");
            
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Neemo", $template, [
                ":INTEGRACAO"    => "Neemo",
                ":NOME_CLIENTE"  => $this->getClientName(),
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
        }
        catch (\Exception $e) {
            $this->util->writeLog("execution", "", "Err: " . $e->getCode() . " - " . $e->getMessage().PHP_EOL);
        }
        $response->withStatus(200);
    }
    
    public function dispatch()
    {
        $pendingOrdersFile = $this->util->getPendingOrdersFile();
        $this->filterDispatchFile($pendingOrdersFile);
        $this->processPendingDispatchOrders($pendingOrdersFile);
        $this->processPendingDeliveredOrders($pendingOrdersFile);
    }
    
    private function filterDispatchFile($pendingOrdersFile)
    {
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData)) {
            return 0;
        }

        $activeList = [];
        foreach ($pendingDispatchData as $key => $currentOrder) {
            
            $diff = (time() - $currentOrder['createdAt']);
            if($diff > 28800) { //8h -> 28800
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $currentOrder['nrComandaExt'] . " expirou (não foi expedido dentro do tempo limite de 8h).\n");
                continue;
            }

            $activeList[$key] = $currentOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    public function processPendingDispatchOrders($pendingOrdersFile)
    {
        
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);
        $pendingDispatchData = json_decode($pendingDispatchFile, true);
        if(empty($pendingDispatchData))
            return 0;

        $cdFilialArr     = [];
        $nrComandaArr    = [];
        $nrComandaExtArr = [];

        foreach($pendingDispatchData as $pendingDispatchOrder) {
            
            if(!empty($pendingDispatchOrder['dispatchedAt']) || $pendingDispatchOrder['delivered']) {
                continue;
            }
            
            if(!isset($pendingDispatchOrder['cdFilial']) || empty($pendingDispatchOrder['cdFilial'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComanda']) || empty($pendingDispatchOrder['nrComanda'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComandaExt']) || empty($pendingDispatchOrder['nrComandaExt'])) {
                continue;                
            }            

            array_push($cdFilialArr,     $pendingDispatchOrder['cdFilial']);
            array_push($nrComandaArr,    $pendingDispatchOrder['nrComanda']);
            array_push($nrComandaExtArr, $pendingDispatchOrder['nrComandaExt']);
        }
        
        if(empty($cdFilialArr))
            return 0;

        if(empty($nrComandaArr))
            return 0;

        if(empty($nrComandaExtArr))
            return 0;
        
        $orders = $this->dispatchService->getPendingDispatchOrders($cdFilialArr, $nrComandaArr, $nrComandaExtArr);
        if(empty($orders))
            return 0; 
        
        foreach($orders as $order) {
            $key = "{$order['CDFILIAL']}|{$order['NRCOMANDA']}|{$order['NRCOMANDAEXT']}";
            
            if($order['IDSTCOMANDA'] == '4') {
                if(isset($pendingDispatchData[$key])) {
                    $reference        = $pendingDispatchData[$key]['reference'];
                    $this->executeWebService("order/".$reference, "PUT", array("status" => "3"));
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " foi cancelado.\n");
                    $pendingDispatchData[$key]['canceled'] = true;
                }
            }
            
            if($order['IDSTCOMANDA'] == '5' || $order['IDSTCOMANDA'] == 'X') {
                if(isset($pendingDispatchData[$key])) {
                    $reference        = $pendingDispatchData[$key]['reference'];
                    $this->executeWebService("order/".$reference, "PUT", array("status" => "4"));
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " saiu para entrega.\n");
                    $pendingDispatchData[$key]['dispatchedAt'] = time();
                }
            }
        }
        
        $activeList = [];
        
        foreach($pendingDispatchData as $key => $currentOrder) {
            if($currentOrder['canceled']) {
                continue;
            }
            $activeList[$key] = $currentOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    public function processPendingDeliveredOrders($pendingOrdersFile)
    {
        $pendingDeliveredFile = file_get_contents($pendingOrdersFile);   
        $pendingDeliveredData = json_decode($pendingDeliveredFile, true);

        if(empty($pendingDeliveredData))
            return 0;

        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            
            if(empty($pendingDeliveredOrder['dispatchedAt'])) {
                continue;
            }

            $diff = (time() - $pendingDeliveredOrder['dispatchedAt']);
            if($diff >= 3600) {
                $reference = $pendingDeliveredOrder['reference'];
                $this->executeWebService("order/".$reference, "PUT", array("status" => "2"));
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDeliveredOrder['nrComandaExt'] . " foi entregue.\n");
                $pendingDeliveredData[$key]['delivered'] = true;
            }
        }
    
        $newPendingList = [];
        
        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            if($pendingDeliveredOrder['delivered']) {
                continue;
            }
            $newPendingList[$key] = $pendingDeliveredOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($newPendingList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    public function processOrder ($orderData)
    {
        $generalParams = $this->integrationService->generalParameters($orderData["branch"]);
        $generalParams["TOGO"] = $orderData["forma_entrega"] == 2 ? true : false;

        $consumerFields = array_combine($this->consumerFieldsList(), array_filter($orderData, function ($assocIndex) {
            return in_array($assocIndex, $this->consumerFieldsList());
        }, ARRAY_FILTER_USE_KEY));
        
        $addressFields = array_combine($this->addressFieldsList(), array_filter($orderData, function ($assocIndex) {
            return in_array($assocIndex, $this->addressFieldsList());
        }, ARRAY_FILTER_USE_KEY));
        
        $consumerDetails = $this->consumerService->processConsumer($generalParams, $orderData);

        $generalParams["CPF"] = "";
        
        return $this->integrationService->openOrderBill($orderData, $generalParams, $consumerDetails);
    }

    public function welcome (Request $request, Response $response)
    {
       
        $sql        = "select * from filial";
        $result     = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();

        return $response->withJson(array(
            'version'      => $versionInfo['version'],
            'revision'     => $versionInfo['revision'],
            'date'         => $versionInfo['date'],
            'filialCount'  => count($result)
        ));
    }

    private function getVersionInfo()
    {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    private function consumerFieldsList ()
    {
        return array(
            "name",
            "email",
            "phone",
            "cpf",
            "client_id"
        );
    }
    
    private function addressFieldsList ()
    {
        return array(
            "street",
            "cep",
            "number",
            "city",
            "neighborhood",
            "complement",
            "reference_point"
        );
    }
    
    private function formatNetworkError($networkData)
    {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }
    
    private function executeWebService($function, $method, $custom_params = [], $generalToken = "")
    {
        $header = [
            "Content-type: application/json"
        ];
        
        if(!empty($generalToken)) {
            $custom_params = array_merge([
                "token_master" => $generalToken //Não compactuamos com este padrão de nomeclatura, mas infelizmente não podemos alterar a API de terceiros -.-'
            ], $custom_params);
        }
        
        $custom_params = array_merge([
            "token_account" => "fixed-token-account",
            "app_token"     => $this->appToken
        ], $custom_params);
        
        
        $body = json_encode($custom_params);
        
        $service_url = self::API_URL . $function;

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $service_url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_FAILONERROR, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        
        $curl_response = curl_exec($curl);
        $httpcode      = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || (stripos($curl_response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->util->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $service_url,
                'method'    => $method,
                'params'    => $custom_params,
                'httpCode'  => $httpcode,
                'error'     => curl_error($curl),
                'response'  => $curl_response
            ]));
        }
       
        curl_close($curl);
        return $curl_response;
    }
    
    private function getClientName()
    {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    public function debugOrder(Request $request, Response $response)
    {
        try {
            $req = $request->getParsedBody();
            
            if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E")
                throw new \Exception("Auth inválido.", 666);

            $configuration = $this->integrationService->getConfiguration();
            
            if(empty($configuration)) {
                $response->withStatus(200);
                return 0;
            }
            
            
            $ufTimezone      = (isset($configuration['IDUFTIMEZONE']) && !empty($configuration['IDUFTIMEZONE'])) ? $configuration['IDUFTIMEZONE'] : "SP";
            $trackingEnabled = (isset($configuration['IDRASTREIAENTREGA']) && ($configuration['IDRASTREIAENTREGA'] == "S"));

            date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));

            if($configuration['IDHABINTE'] == 'N') {
                $response->withStatus(200);
                return 0;
            }
            
            $this->mailList  = $this->util->getMailList();

            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $clientFolder = $this->connection->getParams();
            $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

            $this->basePath          = "/home/developer/workfolder/logs/neemo/PDBORCL";
            $this->ordersPath        = $this->basePath . "/orders";
            $this->processedPath     = $this->basePath . "/processed";
            $this->errPath           = $this->basePath . "/error";
            $this->networkErrPath    = $this->basePath . "/networkError";
            $this->serverName        = "dell";

            if(empty($configuration['IDTOKENINTGERAL'])) {
                throw new Exception\GeneralErrorException("Token geral não encontrado");
            }

            if(empty($configuration['IDSENHATOKENGERAL'])) {
                throw new Exception\GeneralErrorException("App Token não encontrado");
            }
            
            $individualOrder = $req['data']['Order'];
            $individualOrder['payment_method'] = "Dinheir";
            
            ////////////// ORDER FILTER ///////////////

            if (time() - strtotime($individualOrder["date"]) > 28800) {
                return $response->withJson([
                    'success' => false,
                    'message' => "Pedido ignorado pois tem mais de 8 horas"
                ]);
            }// skip orders olders than 8 hours
                
            ////////////// ORDER FILTER ///////////////

            $this->connection->beginTransaction();

            try {
                $fileName =  "INDEFINIDO-" . date("H-i-s") . ".json";

                $cdFilial = "0001";

                $idOrder = $individualOrder['id'];

                $fileName = (isset($idOrder) ? ("$cdFilial-{$idOrder}") : ("INDEFINIDO-" . date("H-i-s"))) . ".json"; 
                
                $this->util->writeLog('order', $fileName, $individualOrder);
                
                $individualOrder["branch"] = "0001";

                $comd = $this->processOrder($individualOrder);
                
                $this->integrationService->validateCashierSinc($comd["CDFILIAL"]);
                
                $this->integrationService->validateOrder($comd['CDFILIAL'], $comd['NRVENDAREST']);
                
                $this->connection->commit();
                
                if($trackingEnabled)
                    $this->dispatchService->registerPendingDispatchOrder($comd["CDFILIAL"], $comd["NRCOMANDA"], $comd["NRCOMANDAEXT"], $individualOrder["id"]);

                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " Pedido - " . $individualOrder["order_number"] . " Integrado com sucesso\n");

                $this->util->writeLog('order_processed', $fileName, $individualOrder);
                
            }
            catch (ApiE $e) {
                $this->connection->rollback();

                $msg = "Não foi possível integrar o pedido - ".$individualOrder["order_number"]."; Motivo: Erro na requisição à API. ErrCode: " . $e->getCode() . " - Message: " . $e->getMessage().PHP_EOL;
                $individualOrder['msg'] = $msg;
                $this->util->writeLog("execution", "", $msg);
                $this->util->writeLog('order_error', $fileName, $individualOrder);
                return $response->withJson([
                    'success' => false,
                    'orderId' => $individualOrder["order_number"]
                ]);
                
            }
            catch (Exception\ParamsException $e) {
                $this->connection->rollback();

                $msg = date("d/m/Y H:i:s") . " Não foi possível integrar o pedido - " .$individualOrder["order_number"].PHP_EOL;
                $msg .= "- Motivo: Erro na requisição à API. ErrCode: " . $e->getCode().PHP_EOL;
                $msg .= "- Message: " . $e->getMessage().PHP_EOL;
                
                $individualOrder['msg'] = $msg;

                $this->util->writeLog('order_error', $fileName, $individualOrder);
                $this->util->writeLog('execution', "", $msg);
                return $response->withJson([
                    'success' => false,
                    'orderId' => $individualOrder["order_number"]
                ]);
            }
            catch (\Exception $e) {
                $this->connection->rollback();

                $msg = date("d/m/Y H:i:s") . " Não foi possível integrar o pedido - " .$individualOrder["order_number"].PHP_EOL;
                $msg .= "- Motivo: Erro na requisição à API. ErrCode: " . $e->getCode().PHP_EOL;
                $msg .= "- Message: " . $e->getMessage().PHP_EOL;

                $individualOrder['msg'] = $msg;

                $this->util->writeLog('order_error', $fileName, $individualOrder);
                $this->util->writeLog('execution', "", $msg);
                return $response->withJson([
                    'success' => false,
                    'orderId' => $individualOrder["order_number"]
                ]);
            }
        }
        catch (ApiE $e) {
            $this->util->writeLog("execution", "", "Erro na requisição à API. ErrCode: " . $e->getCode() . " - Message: " . $e->getMessage().PHP_EOL);
            return $response->withJson([
                'success' => false,
                'orderId' => $individualOrder["order_number"]
            ]);
        }
        
        catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de rede (NetworkException):  {$e->getMessage()}\n");
            return $response->withJson([
                'success' => false,
                'orderId' => $individualOrder["order_number"]
            ]);
        }
        
        catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." " . $e->getMessage() . "\n");
            $this->integrationService->disableIntegration();
            return $response->withJson([
                'success' => false,
                'orderId' => $individualOrder["order_number"]
            ]);
        }
        catch (\Exception $e) {
            $this->util->writeLog("execution", "", "Err: " . $e->getCode() . " - " . $e->getMessage().PHP_EOL);
            return $response->withJson([
                'success' => false,
                'orderId' => $individualOrder["order_number"]
            ]);
        }
        
        return $response->withJson([
            'success' => true,
            'orderId' => $individualOrder["order_number"]
        ]);
    }
}